
clear all
set more off 


***   directories
local dir = "...\replication"


***   =50 if imputed, =0 if not imputed
local thre = 50

***   regressors: censoring probabilities
local var_prob "s_13 x_0_9 x_10_19 x_20_29 x_30 g"

***   regressors: quantile regressions
local var_qreg "s s2 x x2 g"


*=================================================================
* Stage 2: Censored quantile regression with/without imputed data
*=================================================================

*---------------------
* for-loop: years
*---------------------
foreach t of numlist 1979/2012 {
	
	*---------------------
	* for-loop: states
	*---------------------
	foreach j of num 1/50 {
		
		*===========================
		* Read indiviual-level data
		*===========================
		use "`dir'\data\sample_`t'.dta", clear
		
		* Replace missing values with imputed wages.
		if `thre'!=0 {
			drop lnw wgt dc1 dc2
			merge 1:m id using "`dir'\data\est\step0_`t'_`j'.dta"
			drop _merge
		}
		* Drop missing values if not using imputed data.
		if `thre'==0 {
			keep if lnw!=.
		}
		
		* Select state. 
		keep if sid == `j'
		
		*=======================================================================
		* Step 1: Probability of censoring due to minimum wage and top-coding
		*=======================================================================
		if `thre'!=0 {
			merge m:1 year sid `var_prob' using "`dir'\data\prob\pr_`t'_`j'.dta"
		}
		if `thre'==0 {
			merge m:1 year sid `var_prob' using "`dir'\data\prob\prob0\pr_`t'_`j'.dta"
		}
		keep if sid == `j'
		* Probability of not being censored
		gen no_pr_c2 = 1 - pr_c2
		
		* Censoring types
		if `thre'==0 {
			* Probability of being censored
			sum   pr_c1 if d_miss == 0
			local pr_c1_max = r(max)
			sum   pr_c2 if d_miss == 0
			local pr_c2_max = r(max)
			* 4 types of censoring
			g       case_step1 = 1 if `pr_c1_max' == 0 & `pr_c2_max' == 0
			replace case_step1 = 2 if `pr_c1_max' == 0 & `pr_c2_max' ~= 0
			replace case_step1 = 3 if `pr_c1_max' ~= 0 & `pr_c2_max' == 0
			replace case_step1 = 4 if `pr_c1_max' ~= 0 & `pr_c2_max' ~= 0
			label var case_step1 "censoring type"
		}
		
		sum   case_step1
		local v_step1 = r(mean)
		
		if `v_step1' == 1 {
			replace pr_c1 = 0
			replace pr_c2 = 0
		}
		if `v_step1' == 2 {
			replace pr_c1 = 0
		}
		if `v_step1' == 3 {
			replace pr_c2 = 0
		}
		*=======================================================================
		* Steps 2 and 3: Quantile regressions
		*=======================================================================
		*---------------------
		* for-loop: quantiles
		*---------------------
		foreach Q of num 4/97{
		
			local q = `Q' / 100
			
			* Flag for methods used to select the sample
			* - 1: no censoring, 2: step 2, 3: step 3
			gen flagc1 = .
			gen flagc2 = .
			* Final sample used for estimation
			gen set_fin = .
			
			* Subsample selection:
			* The probability of being top-coded is pr_c1(x) := Pr{y_max <= y|x}
			*  For q-th quantile regression drop if (1 - q) <= pr_c1(x).  
			* Forget about censoring if the probability of being top-coded
			*  is zero.
			sum   pr_c1 
			local pr_c1_max = r(max)
			sum   pr_c2 
			local pr_c2_max = r(max)
			
			if `pr_c1_max' == 0 {
				
				replace flagc1 = 1 /* 1: no censoring */
				
				if `pr_c2_max' == 0 {
				
					*-----------------------------------------------------------
					* Case 1: No censoring
					*-----------------------------------------------------------
					replace flagc2  = 1  /* 1: no censoring */
					replace set_fin = 1  /* 1: whole sample */
					
					* Quantile regression w/o censoring using whole sample 
					qreg lnw `var_qreg' [pw = wgt], q(`q') wlsiter(1) 
					
				}
				else {

					*-----------------------------------------------------------
					* Case 2: Right-censoring due to top coding
					*-----------------------------------------------------------
					
					*======================================
					* Step 2: Censored quantile regression
					*======================================
					* Subsample selection 1:
					* The quantile of interest should fall inside the range of
					*  probabilities of not being censored.
					gen set2 = (`q' < no_pr_c2)
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is one,
					*  keep all observations.
					centile no_pr_c2 if set2 == 1, c(10)
					local   no_pc2 = r(c_1)
					
					* Stop dropping observations if the probability of not being
					*  censored is very large above "0.999".
					if `no_pc2' < 0.999 {
						replace set2 = 0 if set2 == 1 & no_pr_c2 < `no_pc2'
					}
				
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2) 
			
					* Fitted values 
					predict double yhat2, xb
			
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 
					replace  resid2 = lnw - lntop if lntop < yhat2 
					gen      vecR2   = ( `q' - 1 * (resid2 <= 0) ) * resid2 
					sum      vecR2 [aw = wgt]  /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*=====================================
					* Step 3: Censored quantile regression
					*=====================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being top-coded) is below 0.97th quantile
					*  in selected sample.
					centile yhat2 if yhat2 < lntop, c(97)
					local   y_cut2 = r(c_1)
					gen     set3   = (yhat2 < `y_cut2') 
					
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if set3 == 1 [pw = wgt], q(`q') wlsiter(1) 
					
					* Subsample
					replace flagc2  = 3     
					replace set_fin = set3
					
					* Fitted values 
					predict double yhat3, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3 
					replace  resid3 = lnw - lntop if lntop < yhat3 
					gen      vecR3   = ( `q' - 1 * (resid3 <= 0) ) * resid3  
					sum      vecR3 [aw = wgt]  /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2. 
					if `obj2' < `obj3' {
				
						replace flagc2  = 2
						replace set_fin = set2
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop set2 set3 yhat2 yhat3
					
				}
			} 
			else {
				
				if `pr_c2_max' == 0 {
			
					*-----------------------------------------------------------
					* Case 3: Left-censoring due to minimum wage
					*-----------------------------------------------------------
					
					*======================================
					* Step 2: Censored quantile regression
					*======================================
					* Subsample selection 1: 
					* The quantile of interest should fall inside the
					*  range of probabilities of not being censored.
					gen set2 = (pr_c1 < `q')
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is one,
					*  keep all observations.
					centile pr_c1 if set2 == 1, c(10)
					local   pc2 = r(c_1)
					
					* Stop dropping observations if the probability of being
					*  censored is very small above "0.001".
					if `pc2' > 0.001 {
						replace set2 = 0 if set2 == 1 & pr_c1 < `pc2'
					}
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2) 
					
					* Fitted values 
					predict double yhat2, xb
					
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 
					replace  resid2 = lnw - lnmw  if yhat2 < lnmw 
					gen      vecR2   = ( `q' - 1 * (resid2 <= 0) ) * resid2  
					sum      vecR2 [aw = wgt]     /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*======================================
					* Step 3: Censored quantile regression
					*======================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being censored at the minimum wage) is
					*  above 0.03th quantile in selected sample.
					centile yhat2 if lnmw < yhat2, c(3)
					local   y_cut2 = r(c_1)
					gen     set3   = (`y_cut2' < yhat2)
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if set3 == 1 [pw = wgt], q(`q') wlsiter(1)
					
					* Subsample 
					replace flagc1  = 3    
					replace flagc2  = 1    
					replace set_fin = set3
					
					* Fitted values 
					predict double yhat3, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3
					replace  resid3 = lnw - lnmw  if yhat3 < lnmw 
					gen      vecR3  = ( `q' - 1 * (resid3 <= 0) ) * resid3 
					sum      vecR3 [aw = wgt]  /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2. 
					if `obj2' < `obj3' {
						
						replace flagc1  = 2    
						replace set_fin = set2
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop set2 set3 yhat2 yhat3
				}
				else {
					
					*-----------------------------------------------------------
					* Case 4: Double censoring
					*-----------------------------------------------------------
					
					*======================================
					* Step 2: Censored Quantile Regression
					*======================================
					* Subsample selection 1:
					* The quantile of interest should fall inside the
					*  range of probabilities of not being censored.
					gen set2 = (pr_c1 < `q' & `q' < no_pr_c2)
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is one,
					*  keep all observations.
					centile pr_c1         if set2 == 1, c(10) /* 10 */
					local   pc2 = r(c_1)  
					centile no_pr_c2      if set2 == 1, c(10) /* 10 */
					local   no_pc2 = r(c_1)
					
					* Stop dropping observations if the probability of not being
					*  censored is very large above "0.999".
					if `no_pc2' < 0.999 {
						replace set2 = 0 if set2 == 1 & no_pr_c2 < `no_pc2'
					}
					* Stop dropping observations if the probability of being
					*  censored is very small above "0.001".
					if `pc2' > 0.001 {
						replace set2 = 0 if set2 == 1 & pr_c1 < `pc2'
					}
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2)
					
					* Fitted values 
					predict double yhat2, xb
					
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 
					replace  resid2 = lnw - lnmw  if yhat2 < lnmw 
					replace  resid2 = lnw - lntop if lntop < yhat2 
					gen      vecR2  = ( `q' - 1 * (resid2 <= 0) ) * resid2 
					sum      vecR2 [aw = wgt]     /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*======================================
					* Step 3: Censored quantile regression
					*======================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being censored) is above 0.03th quantile
					*  and below 0.97th quantile in selected sample.
					centile yhat2 if lnmw < yhat2,  c(3)  /* 3%  */
					local   y_cut3A = r(c_1)
					centile yhat2 if yhat2 < lntop, c(97) /* 97% */
					local   y_cut3B = r(c_1)
					gen     set3 = (`y_cut3A' < yhat2 & yhat2 < `y_cut3B') 
					if `y_cut3A' == `y_cut3B' {
						replace set3 = (lnmw < yhat2 & yhat2 < lntop)
					}
					
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if set3 == 1 [pw = wgt], q(`q') wlsiter(1)
					
					* Subsample 
					replace flagc1  = 3    
					replace flagc2  = 3    
					replace set_fin = set3
					
					* Fitted values 
					predict double yhat3, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3 
					replace  resid3 = lnw - lnmw  if yhat3 < lnmw 
					replace  resid3 = lnw - lntop if lntop < yhat3 
					gen      vecR3   = ( `q' - 1 * (resid3 <= 0) ) * resid3  
					sum      vecR3 [aw = wgt] /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2.
					if `obj2' < `obj3' {
						
						replace flagc1  = 2    
						replace flagc2  = 2    
						replace set_fin = set2
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop set2 set3 yhat2 yhat3
	
				}
			}
			
			*******************
			* Result
			*******************
			* Store quantile regression results.
			predict double lnw_in_q`Q', xb 
			
			qui gen a0_q`Q' = _b[_cons] 
			qui gen a1_q`Q' = _b[s]     
			qui gen a2_q`Q' = _b[s2] 
			qui gen a3_q`Q' = _b[x]     
			qui gen a4_q`Q' = _b[x2]    
			qui gen a5_q`Q' = _b[g]
			
            qui gen  set_fin_q`Q' = set_fin
			qui gen  flagc1_q`Q'  = flagc1
			qui gen  flagc2_q`Q'  = flagc2
			qui egen pr_use_q`Q'  = mean(set_fin)
			
			label var set_fin_q`Q' "Subset used for CQR estimation"
			label var flagc1_q`Q'  "Method 1 QR, 2 CQR, 3 CQR"
			label var flagc2_q`Q'  "Method 1 QR, 2 CQR, 3 CQR"
			label var pr_use_q`Q'  "the portion of observations used for estimation"
			
			drop flagc1 flagc2 set_fin
			
		} /* EoL: quantile */
	
	**********************
	* Save
	**********************
	aorder
	preserve
	if `thre'==0 {
		keep id set_fin_q4-set_fin_q97
		save `dir'\data\est\est0\step1_sample_`t'_`j'.dta, replace
	}
	if `thre'!=0 {
		keep id obs set_fin_q4-set_fin_q97
		save `dir'\data\est\step1_sample_`t'_`j'.dta, replace
	}
	
	restore
	if `thre'==0 {
		collapse sid s s2 x x2 g a0_q4-a5_q97 lnmw (rawsum) sumwt=wgt [pw = wgt], by(state year)
		save "`dir'\data\est\est0\step1_`t'_`j'.dta", replace
	}
	if `thre'!=0 {
		collapse sid s s2 x x2 g a0_q4-a5_q97 lnmw (rawsum) sumwt=wgt [pw = wgt], by(state year)
		save "`dir'\data\est\step1_`t'_`j'.dta", replace
	}
	
	} /* EoL: state */

} /* EoL: time */	


if `thre'==0 {
	forvalues t = 1979/2012 {
		forvalues i = 1/50 {
			if `t'==1979 & `i'==1 {
				use "`dir'\data\est\est0\step1_`t'_`i'.dta"
			}
			else {
				append using "`dir'\data\est\est0\step1_`t'_`i'.dta" 
			}
		}
	}
save "`dir'\data\est\est0\step1_all.dta", replace
}
if `thre'!=0 {
	forvalues t = 1979/2012 {
		forvalues i = 1/50 {
			if `t'==1979 & `i'==1 {
				use "`dir'\data\est\step1_`t'_`i'.dta"
			}
			else {
				append using "`dir'\data\est\step1_`t'_`i'.dta" 
			}
		}
	}
save "`dir'\data\est\step1_all.dta", replace
}

if `thre'!=0 {
	forvalues t = 1979/2012 {
		forvalues i = 1/50 {
			erase "`dir'\data\est\step1_`t'_`i'.dta"
		}
	}
}
if `thre'==0 {
	forvalues t = 1979/2012 {
		forvalues i = 1/50 {
			erase "`dir'\data\est\est0\step1_`t'_`i'.dta"
		}
	}
}
