%==========================================================================
% Boostrap: 2nd stage with lead and lag  
%--------------------------------------------------------------------------
% - Bootstrap for State-level panel data regression 
%==========================================================================
clc; clear;
 
%==========================================================================
% setup 
%==========================================================================
% (1) imputation set up (0 or 50)
spec = 50;     

% (2) the number of resamples at the first stage (500 in our paper)
Nboot1 = 500;   

% (3) the number of resamples at the second stage (1000 in our paper)
Nboot2 = 1000;  

% (4) working directory 
%  -  set working directory the location of this Matlab script. 

%---------------------------------------------------------------------------
% Please do not change the scripts from here. 
%---------------------------------------------------------------------------
bigK  = 6;      % # regressors 
bigS  = 50;     % # state 
bigT  = 34 - 2; % # years (-2 b/c leand and lag)
bigQ  = 94;     % # quantiles 
bigST = bigS * bigT;
bigKQ = bigK * bigQ;

%==========================================================================
% (A) Data  
%==========================================================================
%--------------------------------------------------------------------------
% (a) State-level Data 
%--------------------------------------------------------------------------
Dname0 = sprintf('../data/boot/boot%d/', spec);

%----------------
% (a1) data
%----------------
DATA0  = csvread( strcat(Dname0, "mw.csv"), 1);
% - 1: year
% - 2: state
% - 3: log min wage (real)
% - 4: sumwt	

DATA1 = sortrows(DATA0,[2 1]);
% note: I sort data because this file should be ordered in state and then year. 
%       Make sure the order is good. 
vecT0 = DATA1(:,1);
vecS0 = DATA1(:,2);
logMW = DATA1(:,3);     % real
vecW0 = DATA1(:,4);

vecSel = (1980 <= vecT0 & vecT0 <= 2011);
vecMW0 = logMW( (vecT0 <= 2010) );                 % 1979-2010: lag
vecMW1 = logMW(  vecSel );                         % 1980-2011
vecMW2 = logMW( (1981 <= vecT0) );                 % 1981-2012: lead 
matMW  = [vecMW0 vecMW1 vecMW2];
vecW   = vecW0(vecSel);                            %1980-2011

%------------------
% (a2) regressors   
%------------------
% Note: The order in DATA0 is very important to construct regressors. The 
% current data is sored by state and time.     

% state fixed effects 
% - assign a blcok vector (T x 1) for each state
matFE = kron( eye(bigS), ones(bigT,1) ); % (S * T) x S = (S x S) ** T x 1

% time effects 
% - assign an identifity matrix (T x T) for each state
matTE = kron( ones(bigS,1), eye(bigT) ); % T*S x T = (S x S) ** T x 1

% state-level trend 
% - a linear trend for each state
matTR = kron( eye(bigS), (1:1:bigT)' ); % S *T x S = (S x S) ** (T x 1)

% - - two regressors should be dropped after checking rank condition 
matFTall = [matFE matTE(:,2:end) matTR(:,2:end)];
if rank(matFTall) ~= size(matFTall,2)
    STOP
end

% clear
clear matFE matTE matTR matFEall
%==========================================================================
% Panel regression with bootstraped estimates    
%==========================================================================
%------------------
% Data combination
%------------------
matY = zeros(bigST, Nboot1*bigKQ);    % container 
%A matW = zeros(bigST, Nboot1);       % container
for s = 1:50   
    for t = 1980:2011  
       
        % read data
        Fname1    = sprintf('bootstrap_s%d_%d.csv', s, t); 
        Data_temp = csvread(  strcat(Dname0, Fname1) ,1); % B x KQ

        % row
        r0 = (t - 1979) + bigT * (s - 1); % <= order is important 

        % reshape
        % - transponse (KQ x BT) => reshape (KQB x T) => tranpose
        matY(r0,:) = reshape(Data_temp(:,3:end-1)', Nboot1*bigKQ, 1)'; % 1 x KQB
        % - - 1st tranpose necessary b/c reshape is column operation
        % - - 2nd tranpose necessary b/c keep colum vector for dependent v.
    end
end     

%-----------------------------------
% Step 2: regression: bootstrap 
%-----------------------------------
rseed = RandStream('mt19937ar','Seed',2016);
RandStream.setGlobalStream(rseed);

tic
matOut0 = zeros(Nboot1*Nboot2, bigKQ); % B1*B2 x KQ
matOut1 = zeros(Nboot1*Nboot2, bigKQ); % B1*B2 x KQ
matOut2 = zeros(Nboot1*Nboot2, bigKQ); % B1*B2 x KQ
for b1 = 1:Nboot1 
    
    % dependent variables
    c0    = 1 + bigKQ * (b1 - 1);
    c1    =     bigKQ *  b1;
    tempY = matY(:,c0:c1);      % ST x KQ
    
    %----------------------------
    % cross-sectional bootstrap
    %----------------------------
    for b2 = 1:Nboot2
    
        %-----------------------------------
        % Step1: randomly selection indices
        %-----------------------------------
        % - starting point: <<< be careful about this part
        vecR_B = randi([1 bigS], bigS, 1);                      % S  x 1
        vecI_B = kron( (1 + bigT * (vecR_B - 1)), ones(bigT, 1) ) ....
                + kron( ones(bigS, 1), (0:1:bigT-1)'); % ST x 1
        
        %-----------------------------------
        % Step 2: resample
        %-----------------------------------
        % state level
        matX_B = [matMW(vecI_B,:), matFTall];
        matW_B = diag( vecW(vecI_B) );
        matY_B = tempY(vecI_B,:);        % ST x KQ
        
        %-----------------------------------
        % Step 3: estimation
        %-----------------------------------
        matEst = ( matX_B' * matW_B * matX_B) \ (matX_B' * matW_B * matY_B);  % #reg x KQ
        
        % store results 
        b = b2 + Nboot2 *(b1 - 1); 
        matOut0(b,:) = matEst(1,:); % (1 x KQ)' 
        matOut1(b,:) = matEst(2,:); % (1 x KQ)' 
        matOut2(b,:) = matEst(3,:); % (1 x KQ)' 

    end 
end 

toc 

%--------------------
% save 
%--------------------
Fname_out0 = sprintf('bootstrap_final_LL0.csv'); 
Fname_out1 = sprintf('bootstrap_final_LL1.csv'); 
Fname_out2 = sprintf('bootstrap_final_LL2.csv'); 
csvwrite( strcat(Dname0, Fname_out0), matOut0);
csvwrite( strcat(Dname0, Fname_out1), matOut1);
csvwrite( strcat(Dname0, Fname_out2), matOut2);
